@@ -144,6 +144,10 @@ SpSubview<eT>::operator*=(const eT val)
   {
   arma_extra_debug_sigprint();
   
+  if(val == eT(0))  { (*this).zeros(); return *this; }
+  
+  if((n_elem == 0) || (n_nonzero == 0))  { return *this; }
+  
   m.sync_csc();
   m.invalidate_cache();
   
@@ -263,191 +267,11 @@ SpSubview<eT>::operator=(const Base<eT, T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  // this is a modified version of SpSubview::operator_equ_common(const SpBase)
+  const quasi_unwrap<T1> U(in.get_ref());
   
-  const SpProxy< SpMat<eT> > pa((*this).m);
+  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
-  const unwrap<T1>     b_tmp(in.get_ref());
-  const Mat<eT>&   b = b_tmp.M;
-  
-  arma_debug_assert_same_size(n_rows, n_cols, b.n_rows, b.n_cols, "insertion into sparse submatrix");
-  
-  const uword pa_start_row = (*this).aux_row1;
-  const uword pa_start_col = (*this).aux_col1;
-  
-  const uword pa_end_row = pa_start_row + (*this).n_rows - 1;
-  const uword pa_end_col = pa_start_col + (*this).n_cols - 1;
-  
-  const uword pa_n_rows = pa.get_n_rows();
-  
-  const uword b_n_elem = b.n_elem;
-  const eT*   b_mem    = b.memptr();
-  
-  uword box_count = 0;
-  
-  for(uword i=0; i<b_n_elem; ++i)
-    {
-    box_count += (b_mem[i] != eT(0)) ? uword(1) : uword(0);
-    }
-  
-  const uword alt_count = pa.get_n_nonzero() - (*this).n_nonzero + box_count;
-  
-  // Resize memory to correct size.
-  SpMat<eT> out(arma_reserve_indicator(), pa.get_n_rows(), pa.get_n_cols(), alt_count);
-  
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_it  = pa.begin();
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_end = pa.end();
-  
-  uword b_row = 0;
-  uword b_col = 0;
-    
-  bool x_it_ok = (x_it != x_end);
-  bool y_it_ok = ( (b_row < b.n_rows) && (b_col < b.n_cols) );
-  
-  uword x_it_row = (x_it_ok) ? x_it.row() : 0;
-  uword x_it_col = (x_it_ok) ? x_it.col() : 0;
-  
-  uword y_it_row = (y_it_ok) ? b_row + pa_start_row : 0;
-  uword y_it_col = (y_it_ok) ? b_col + pa_start_col : 0;
-    
-  uword cur_val = 0;
-  while(x_it_ok || y_it_ok)
-    {
-    const bool x_inside_box = (x_it_row >= pa_start_row) && (x_it_row <= pa_end_row) && (x_it_col >= pa_start_col) && (x_it_col <= pa_end_col);
-    const bool y_inside_box = (y_it_row >= pa_start_row) && (y_it_row <= pa_end_row) && (y_it_col >= pa_start_col) && (y_it_col <= pa_end_col);
-    
-    const eT x_val = x_inside_box ? eT(0) : ( x_it_ok ? (*x_it) : eT(0) );
-    
-    const eT y_val = y_inside_box ? ( y_it_ok ? b.at(b_row,b_col) : eT(0) ) : eT(0);
-    
-    if( (x_it_row == y_it_row) && (x_it_col == y_it_col) )
-      {
-      if( (x_val != eT(0)) || (y_val != eT(0)) )  
-        {
-        access::rw(out.values[cur_val]) = (x_val != eT(0)) ? x_val : y_val;
-        access::rw(out.row_indices[cur_val]) = x_it_row;
-        ++access::rw(out.col_ptrs[x_it_col + 1]);
-        ++cur_val;
-        }
-      
-      if(x_it_ok)
-        {
-        ++x_it;
-        
-        if(x_it == x_end)  { x_it_ok = false; }
-        }
-      
-      if(x_it_ok)
-        {
-        x_it_row = x_it.row();
-        x_it_col = x_it.col();
-        }
-      else
-        {
-        x_it_row++;
-        
-        if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-        }
-      
-      if(y_it_ok)
-        {
-        b_row++;
-        
-        if(b_row >= b.n_rows)  { b_row = 0; b_col++; }
-        
-        if( (b_row > b.n_rows) || (b_col > b.n_cols) )  { y_it_ok = false; }
-        }
-      
-      if(y_it_ok)
-        {
-        y_it_row = b_row + pa_start_row;
-        y_it_col = b_col + pa_start_col;
-        }
-      else
-        {
-        y_it_row++;
-        
-        if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-        }
-      }
-    else
-      {
-      if((x_it_col < y_it_col) || ((x_it_col == y_it_col) && (x_it_row < y_it_row))) // if y is closer to the end
-        {
-        if(x_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = x_val;
-          access::rw(out.row_indices[cur_val]) = x_it_row;
-          ++access::rw(out.col_ptrs[x_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(x_it_ok)
-          {
-          ++x_it;
-          
-          if(x_it == x_end)  { x_it_ok = false; }
-          }
-        
-        if(x_it_ok)
-          {
-          x_it_row = x_it.row();
-          x_it_col = x_it.col();
-          }
-        else
-          {
-          x_it_row++;
-          
-          if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-          }
-        }
-      else
-        {
-        if(y_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = y_val;
-          access::rw(out.row_indices[cur_val]) = y_it_row;
-          ++access::rw(out.col_ptrs[y_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(y_it_ok)
-          {
-          b_row++;
-          
-          if(b_row >= b.n_rows)  { b_row = 0; b_col++; }
-          
-          if( (b_row > b.n_rows) || (b_col > b.n_cols) )  { y_it_ok = false; }
-          }
-        
-        if(y_it_ok)
-          {
-          y_it_row = b_row + pa_start_row;
-          y_it_col = b_col + pa_start_col;
-          }
-        else
-          {
-          y_it_row++;
-          
-          if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-          }
-        }
-      }
-    }
-  
-  const uword out_n_cols = out.n_cols;
-  
-  uword* col_ptrs = access::rwp(out.col_ptrs);
-  
-  // Fix column pointers to be cumulative.
-  for(uword c = 1; c <= out_n_cols; ++c)
-    {
-    col_ptrs[c] += col_ptrs[c - 1];
-    }
-  
-  access::rw((*this).m).steal_mem(out);
-  
-  access::rw(n_nonzero) = box_count;
+  spglue_merge::subview_merge(*this, U.M);
   
   return *this;
   }
@@ -556,186 +380,18 @@ SpSubview<eT>::operator_equ_common(const SpBase<eT, T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  // algorithm:
-  // instead of directly inserting values into the matrix underlying the subview,
-  // create a new matrix by merging the underlying matrix with the input object,
-  // and then replacing the underlying matrix with the created matrix.
-  // 
-  // the merging process requires pretending that the input object
-  // has the same size as the underlying matrix.
-  // while iterating through the elements of the input object,
-  // this requires adjusting the row and column locations of each element,
-  // as well as providing fake zero elements.
-  // in effect there is a proxy for a proxy.
+  const unwrap_spmat<T1> U(in.get_ref());
   
-  
-  const SpProxy< SpMat<eT> > pa((*this).m   );
-  const SpProxy< T1        > pb(in.get_ref());
-  
-  arma_debug_assert_same_size(n_rows, n_cols, pb.get_n_rows(), pb.get_n_cols(), "insertion into sparse submatrix");
-  
-  const uword pa_start_row = (*this).aux_row1;
-  const uword pa_start_col = (*this).aux_col1;
-  
-  const uword pa_end_row = pa_start_row + (*this).n_rows - 1;
-  const uword pa_end_col = pa_start_col + (*this).n_cols - 1;
-  
-  const uword pa_n_rows = pa.get_n_rows();
-  
-  const uword alt_count = pa.get_n_nonzero() - (*this).n_nonzero + pb.get_n_nonzero();
-  
-  // Resize memory to correct size.
-  SpMat<eT> out(arma_reserve_indicator(), pa.get_n_rows(), pa.get_n_cols(), alt_count);
-  
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_it  = pa.begin();
-  typename SpProxy< SpMat<eT> >::const_iterator_type x_end = pa.end();
-  
-  typename SpProxy<T1>::const_iterator_type y_it  = pb.begin();
-  typename SpProxy<T1>::const_iterator_type y_end = pb.end();
-  
-  bool x_it_ok = (x_it != x_end);
-  bool y_it_ok = (y_it != y_end);
-  
-  uword x_it_row = (x_it_ok) ? x_it.row() : 0;
-  uword x_it_col = (x_it_ok) ? x_it.col() : 0;
-  
-  uword y_it_row = (y_it_ok) ? y_it.row() + pa_start_row : 0;
-  uword y_it_col = (y_it_ok) ? y_it.col() + pa_start_col : 0;
-    
-  uword cur_val = 0;
-  while(x_it_ok || y_it_ok)
+  if(U.is_alias(m))
     {
-    const bool x_inside_box = (x_it_row >= pa_start_row) && (x_it_row <= pa_end_row) && (x_it_col >= pa_start_col) && (x_it_col <= pa_end_col);
-    const bool y_inside_box = (y_it_row >= pa_start_row) && (y_it_row <= pa_end_row) && (y_it_col >= pa_start_col) && (y_it_col <= pa_end_col);
+    const SpMat<eT> tmp(U.M);
     
-    const eT x_val = x_inside_box ? eT(0) : ( x_it_ok ? (*x_it) : eT(0) );
-    
-    const eT y_val = y_inside_box ? ( y_it_ok ? (*y_it) : eT(0) ) : eT(0);
-    
-    if( (x_it_row == y_it_row) && (x_it_col == y_it_col) )
-      {
-      if( (x_val != eT(0)) || (y_val != eT(0)) )  
-        {
-        access::rw(out.values[cur_val]) = (x_val != eT(0)) ? x_val : y_val;
-        access::rw(out.row_indices[cur_val]) = x_it_row;
-        ++access::rw(out.col_ptrs[x_it_col + 1]);
-        ++cur_val;
-        }
-      
-      if(x_it_ok)
-        {
-        ++x_it;
-        
-        if(x_it == x_end)  { x_it_ok = false; }
-        }
-      
-      if(x_it_ok)
-        {
-        x_it_row = x_it.row();
-        x_it_col = x_it.col();
-        }
-      else
-        {
-        x_it_row++;
-        
-        if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-        }
-      
-      if(y_it_ok)
-        {
-        ++y_it;
-        
-        if(y_it == y_end)  { y_it_ok = false; }
-        }
-      
-      if(y_it_ok)
-        {
-        y_it_row = y_it.row() + pa_start_row;
-        y_it_col = y_it.col() + pa_start_col;
-        }
-      else
-        {
-        y_it_row++;
-        
-        if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-        }
-      }
-    else
-      {
-      if((x_it_col < y_it_col) || ((x_it_col == y_it_col) && (x_it_row < y_it_row))) // if y is closer to the end
-        {
-        if(x_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = x_val;
-          access::rw(out.row_indices[cur_val]) = x_it_row;
-          ++access::rw(out.col_ptrs[x_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(x_it_ok)
-          {
-          ++x_it;
-          
-          if(x_it == x_end)  { x_it_ok = false; }
-          }
-        
-        if(x_it_ok)
-          {
-          x_it_row = x_it.row();
-          x_it_col = x_it.col();
-          }
-        else
-          {
-          x_it_row++;
-          
-          if(x_it_row >= pa_n_rows)  { x_it_row = 0; x_it_col++; }
-          }
-        }
-      else
-        {
-        if(y_val != eT(0))
-          {
-          access::rw(out.values[cur_val]) = y_val;
-          access::rw(out.row_indices[cur_val]) = y_it_row;
-          ++access::rw(out.col_ptrs[y_it_col + 1]);
-          ++cur_val;
-          }
-        
-        if(y_it_ok)
-          {
-          ++y_it;
-          
-          if(y_it == y_end)  { y_it_ok = false; }
-          }
-        
-        if(y_it_ok)
-          {
-          y_it_row = y_it.row() + pa_start_row;
-          y_it_col = y_it.col() + pa_start_col;
-          }
-        else
-          {
-          y_it_row++;
-          
-          if(y_it_row >= pa_n_rows)  { y_it_row = 0; y_it_col++; }
-          }
-        }
-      }
+    return (*this).operator_equ_common(tmp);
     }
   
-  const uword out_n_cols = out.n_cols;
+  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
-  uword* col_ptrs = access::rwp(out.col_ptrs);
-  
-  // Fix column pointers to be cumulative.
-  for(uword c = 1; c <= out_n_cols; ++c)
-    {
-    col_ptrs[c] += col_ptrs[c - 1];
-    }
-  
-  access::rw((*this).m).steal_mem(out);
-  
-  access::rw(n_nonzero) = pb.get_n_nonzero();
+  spglue_merge::subview_merge(*this, U.M);
   
   return *this;
   }
@@ -1062,6 +718,34 @@ SpSubview<eT>::replace(const eT old_val, const eT new_val)
 
 
 
+template<typename eT>
+inline
+void
+SpSubview<eT>::clean(const typename get_pod_type<eT>::result threshold)
+  {
+  arma_extra_debug_sigprint();
+  
+  if((n_elem == 0) || (n_nonzero == 0))  { return; }
+  
+  // TODO: replace with a more efficient implementation
+  
+  SpMat<eT> tmp(*this);
+  
+  tmp.clean(threshold);
+  
+  if(is_cx<eT>::yes)
+    {
+    (*this).operator=(tmp);
+    }
+  else
+  if(tmp.n_nonzero != n_nonzero)
+    {
+    (*this).operator=(tmp);
+    }
+  }
+
+
+
 template<typename eT>
 inline
 void
@@ -1092,7 +776,52 @@ SpSubview<eT>::zeros()
   {
   arma_extra_debug_sigprint();
   
-  (*this).operator*=(eT(0));
+  if((n_elem == 0) || (n_nonzero == 0))  { return; }
+  
+  if((m.n_nonzero - n_nonzero) == 0)
+    {
+    access::rw(m).zeros();
+    access::rw(n_nonzero) = 0;
+    return;
+    }
+  
+  SpMat<eT> tmp(arma_reserve_indicator(), m.n_rows, m.n_cols, m.n_nonzero - n_nonzero);
+  
+  const uword sv_row_start = aux_row1;
+  const uword sv_col_start = aux_col1;
+  
+  const uword sv_row_end   = aux_row1 + n_rows - 1;
+  const uword sv_col_end   = aux_col1 + n_cols - 1;
+  
+  typename SpMat<eT>::const_iterator m_it     = m.begin();
+  typename SpMat<eT>::const_iterator m_it_end = m.end();
+  
+  uword tmp_count = 0;
+  
+  for(; m_it != m_it_end; ++m_it)
+    {
+    const uword m_it_row = m_it.row();
+    const uword m_it_col = m_it.col();
+    
+    const bool inside_box = ((m_it_row >= sv_row_start) && (m_it_row <= sv_row_end)) && ((m_it_col >= sv_col_start) && (m_it_col <= sv_col_end));
+    
+    if(inside_box == false)
+      {
+      access::rw(tmp.values[tmp_count])      = (*m_it);
+      access::rw(tmp.row_indices[tmp_count]) = m_it_row;
+      access::rw(tmp.col_ptrs[m_it_col + 1])++;
+      ++tmp_count;
+      }
+    }
+  
+  for(uword i=0; i < tmp.n_cols; ++i)
+    {
+    access::rw(tmp.col_ptrs[i + 1]) += tmp.col_ptrs[i];
+    }
+  
+  access::rw(m).steal_mem(tmp);
+  
+  access::rw(n_nonzero) = 0;
   }
 
 
